"use strict";

/**
 * 获取认证参数
 * @param aac002 身份证号 无卡认证必传
 * @return {{dev_no: string}}
 */
function getAuthParam(aac002) {
    var param = {
        dev_no: Base.getYNCHSDevNo()
    };
    if (aac002) {
        param.aac002 = aac002;
    }
    return param;
}

/**
 * readCard 读取卡信息
 * @param {Object} authParam 额外认证参数
 * @param {String} authParam.dev_no 设备编号
 * @param callback(ret-readCard(1193)返回ret, err-readCard(1193)返回err, params-回调函数额外业务参数) 回调函数
 * @param params 回调函数额外业务参数
 */
function ChsReadCard(authParam, callback, params) {
    var param = {};
	param["SERVICENAME"] =  'httpService.getYNCHSYnhsdata';  //接口
	param["jybh"] = 'HISCCT001';
	Base.requestCct("queryData", param, function(data){
		console.log("" + JSON.stringify(data));
		if(data.flag == "1") {
			var readCardParam = {};
			readCardParam.type = "001";
			readCardParam.request = data.result;
			readCardParam.request.ynhsdata.infno = "1193";
			readCardParam.request.ynhsdata.dev_no = authParam.dev_no;
			readCardParam.request.ynhsdata.input = {
			    data: {
			        idno: "",
			        username: "",
			        mdtrtarea: Base.getStorage("mdtrtarea_admvs")
			    }
			};
			console.log('开始新医保读卡-1193');
			Base.console('新医保读卡入参', readCardParam);
			console.log("新医保读卡入参" +JSON.stringify(readCardParam));
			ynhsa.readCard(readCardParam, function (ret, err) {
			    Base.console('新医保读卡-ret', ret);
			    Base.console('新医保读卡-err', err);
			    if (callback) {
			        callback(ret, err, params);
			    }
			});
		}
	},true);
    /* Base.request("httpService.getYNCHSYnhsdata", {}, function (data) {
		// 读卡认证
		var readCardParam = {};
		readCardParam.type = "001";
        Base.requestIsSuccess(data, function (success) {
            readCardParam.request = success.result;
            readCardParam.request.ynhsdata.infno = "1193";
            readCardParam.request.ynhsdata.dev_no = authParam.dev_no;
            readCardParam.request.ynhsdata.input = {
                data: {
                    idno: "",
                    username: "",
                    mdtrtarea: Base.getStorage("mdtrtarea_admvs")
                }
            };
            Base.console('开始新医保读卡-1193');
            Base.console('新医保读卡入参', readCardParam);
            ynhsa.readCard(readCardParam, function (ret, err) {
                Base.console('新医保读卡-ret', ret);
                Base.console('新医保读卡-err', err);
                if (callback) {
                    callback(ret, err, params);
                }
            });
        })
    }, "正在获取刷卡设备信息..."); */
}

/**
 * readCard 读取卡信息 成功后 直接调用chsApi 读卡认证 1191
 * @param authParam 读卡认证额外参数
 * @param callback (ret-chsApi(1191)返回ret, err-chsApi(1191)返回err, readCardResult-readCard(1193)返回, param-回调函数所需业务参数)
 * @param params
 */
function readCardSuccessToAuth(authParam, params, callback) {
    // ynhsa.getDeviceId({type: "001"}, function (devRet) {
        // authParam.dev_no = devRet.deviceId;
		authParam.dev_no = authParam.dev_no;
        ChsReadCard(authParam, function (ret, err) {
            ChsReadCardHandle(ret, err, function () {
                ChsReadCardAuth(authParam, ret, function (ret, err, readCardResult, params) {
                    ChsReadCardAuthSuccessHandle(ret, err, readCardResult, callback, params);
                }, params);
            });
        });
    // });
}

/**
 * readCard 读取卡信息 完成后的公共处理 也可自行处理
 * 读卡成功 调用回调方法
 * 读卡失败 弹出错误提示
 * @param ret readCard(1193)返回ret
 * @param err readCard(1193)返回err
 * @param callback(ret-chsApi(1191)返回ret, err-chsApi(1191)返回err)
 */
function ChsReadCardHandle(ret, err, callback) {
    if (ChsReadCardSuccess(ret, err)) {
        if (callback) {
            callback(ret, err);
        }
    } else {
        var errMsg = err.message;
        if (errMsg === '无卡') {
            errMsg = '读卡失败，请插入社保卡';
        } else {
            errMsg = '读卡失败：' + errMsg;
        }
        Base.msgTip(errMsg, 5000);
    }
}

/**
 * readCard 读取卡信息 是否成功
 * @param ret readCard(1193)返回ret
 * @param err readCard(1193)返回err
 * @return boolean
 */
function ChsReadCardSuccess(ret, err) {
    return !!Base.isNotEmpty(ret);
}

/**
 * chsApi 1191 是否成功
 * @param ret chsApi(1191)返回ret
 * @param err chsApi(1191)返回err
 * @return {boolean}
 */
function ChsReadCardAuthSuccess(ret, err) {
    return !!(Base.isNotEmpty(ret) && ret.data && ret.data.aae314 == '1');
}

/**
 * chsApi 1191 错误信息获取
 * @return {string}
 */
function ChsReadCardAuthFailMsg(ret, err) {
    var errMsg = '';
    if (Base.isNotEmpty(ret) && ret.data) {
        errMsg = ret.data.aae204
    } else if (Base.isNotEmpty(err)) {
        errMsg = err.message;
    }
    return errMsg;
}

/**
 * chsApi 1191 默认处理
 * @param ret chsApi 1191 ret
 * @param err chsApi 1191 err
 * @param readCardResult readCard(1193)返回ret
 * @param callback(ret-chsApi(1191)返回ret, err-chsApi(1191)返回err, readCardResult-readCard(1193)返回ret, params-回调函数额外业务参数) 回调函数
 * @param params 回调函数参数
 */
function ChsReadCardAuthSuccessHandle(ret, err, readCardResult, callback, params) {
    if (ChsReadCardAuthSuccess(ret, err)) {
        if (callback) {
            callback(ret, err, readCardResult, params)
        }
    } else {
        Base.msgTip('身份认证失败：' + ChsReadCardAuthFailMsg(ret, err));
    }
}

/**
 * chsApi 读卡认证 1191
 * @param {Object} authParam 额外认证参数
 * @param {String} authParam.dev_no 设备编号
 * @param {Object} readCardResult readCard(1193) 返回
 * @param readCardResult.cardno 卡号 字符型 50
 * @param readCardResult.identifyCardNo 卡识别码 字符型 50
 * @param readCardResult.personno 人员编号 字符型 32
 * @param readCardResult.initno 字符型
 * @param readCardResult.name 持卡人姓名 字符型 6
 * @param readCardResult.nation 国家代码 字符型 50 Y
 * @param readCardResult.identifyno 证件号码 字符型 50
 * @param readCardResult.birthdate 出生日期 字符型 50 YYYYMMDD
 * @param readCardResult.sex 性别 字符型 50 Y
 * @param readCardResult.unitno 字符型 100
 * @param readCardResult.status 状态 字符型 50 Y
 * @param readCardResult.Balance 字符型
 * @param readCardResult.Password 密码 字符型
 * @param readCardResult.standardVersion 卡版本 字符型
 * @param readCardResult.ylcbd 医疗参保地 字符型 6 Y
 * @param readCardResult.by1 备用字段
 * @param callback(ret, err, readCardResult, params) 回调函数
 * @param params 回调函数额外业务参数
 */
function ChsReadCardAuth(authParam, readCardResult, callback, params) {

    // aac002 社会保障号码 字符型 50 Y
    // 2 aaz500 卡号 字符型 50 Y
    // 3 aaz501 卡识别码 字符型 50 Y
    // 4 aac003 姓名 字符型 20 Y
    // 5 baz821 卡规范版本 字符型 10 Y 1.0：一代卡 2.00：二代卡 3.00：三代卡
    // 6 carttype 读卡类型 字符型 20 Y 0：实体卡 1：电子社保卡
    // 7 areacode 发卡地区行政区划码 字符型 20 卡识别码前6位
    // 8 isverypin 是否验证 pin 字符型 31 验证-1 未验证-0
	var param = {};
	param["SERVICENAME"] =  'httpService.getYNCHSYnhsdata';  //接口
	param["jybh"] = 'HISCCT001';
	Base.requestCct("queryData", param, function(data){
		console.log("data: " + JSON.stringify(data));
		if(data.flag == "1") {
			var chsApiParam = {};
			chsApiParam.request = {};
			chsApiParam.request = data.result;
			chsApiParam.request.ynhsdata.infno = "1191";
			chsApiParam.request.ynhsdata.dev_no = authParam.dev_no;
			var insuplc_admdvs = readCardResult.identifyCardNo.substr(0, 6);
			chsApiParam.request.ynhsdata.insuplc_admdvs = insuplc_admdvs;
			chsApiParam.request.ynhsdata.input = {
			    cardinfo: {
			        aac002: readCardResult.identifyno, // 社会保障号码  identifyno-证件号码
			        aaz500: readCardResult.cardno,// 卡号 cardno-卡号
			        aaz501: readCardResult.identifyCardNo,// 卡识别码 identifyCardNo-卡识别码
			        aac003: readCardResult.name,// 姓名 name-持卡人姓名
			        baz821: readCardResult.standardVersion,// 卡规范版本[1.0：一代卡 2.00：二代卡 3.00：三代卡] standardVersion-卡版本
			        carttype: '0',// 读卡类型[0：实体卡  1：电子社保卡] 1191出参无 暂定0
			        areacode: insuplc_admdvs,// areacode 发卡地区行政区划码 卡识别码前6位
			        isverypin: '0'// 是否验证 pin 验证-1 未验证-0 1191出参无 暂定1
			    }
			};
			Base.console('开始新医保读卡认证-1191');
			Base.console('新医保读卡认证-1191入参', chsApiParam);
			ynhsa.chsApi(chsApiParam, function (ret, err) {
			    Base.console('新医保读卡认证-1191-ret', ret);
			    Base.console('新医保读卡认证-1191-err', err);
			    if (callback) {
			        callback(ret, err, readCardResult, params);
			    }
			})
		}
	},true);
    /* Base.request("httpService.getYNCHSYnhsdata", {}, function (data) {
        Base.requestIsSuccess(data, function (success) {

        })
    }, "正在获取刷卡设备信息..."); */
}

/**
 * chsApiPwdValid(1193) 身份证+密码认证 是否成功
 * @param ret chsApiPwdValid(1193) 身份证+密码认证返回ret
 * @param err chsApiPwdValid(1193) 身份证+密码认证返回err
 * @return boolean
 */
function ChsIdNoAuthSuccess(ret, err) {
    return !!(ret && ret.data && ret.data.flag == '1');
}

/**
 * chsApiPwdValid(1193) 身份证+密码认证
 * @param {Object} authParam 额外认证参数
 * @param {string} authParam.dev_no 设备编号
 * @param {string} authParam.aac002 身份证号
 * @param callback (ret-chsApiPwdValid(1193) 身份证+密码认证返回ret, err-chsApiPwdValid(1193) 身份证+密码认证返回err, params)回调函数
 * @param params 回调函数额外业务参数
 */
function ChsIdNoAuth(authParam, callback, params) {
    // 1193 chsApiPwdValid

	var param = {};
	param["SERVICENAME"] =  'httpService.getYNCHSYnhsdata';  //接口
	param["jybh"] = 'HISCCT001';
	Base.requestCct("queryData", param, function(data){
		console.log("data: " + JSON.stringify(data));
		if(data.flag == "1") {
			var chsParam = {};
			chsParam.type = "001";
			chsParam.request = data.result;
			chsParam.request.ynhsdata.infno = "1193";
			chsParam.request.ynhsdata.dev_no = authParam.dev_no;
			chsParam.request.ynhsdata.input = {
			    data: {
			        idno: authParam.aac002,
			        username: "",
			        mdtrtarea: Base.getStorage("mdtrtarea_admvs")
			    }
			};
			Base.console('开始新医保无卡认证');
			Base.console('新医保无卡认证入参', chsParam);
			ynhsa.chsApiPwdValid(chsParam, function (ret, err) {
			    Base.console('新医保无卡认证-ret', ret);
			    Base.console('新医保无卡认证-err', err);
			    if (callback) {
			        callback(ret, err, params);
			    }
			});
		}
	});
}

/**
 * chsApi 身份核验 包括 1192 1194
 * @param verify 核验参数 自行组装 1192或1194入参
 * @param infno 交易编号
 * @param dev_no 设备编号
 * @param callback (ret-chsApi身份核验返回ret, err-chsApi身份核验返回err, params)回调函数
 * @param params 回调函数额外业务参数
 * @param failRepeat 核验失败是否重新认证 外部调用时必须传true 内部递归调用时传false 用于控制最多重新认证核验一次
 * @param repeatParam 重新认证核验所需参数
 */
function ChsAuthenticateVerify(verify, infno, dev_no, callback, params, failRepeat, repeatParam) {
    var param = {};
    param["SERVICENAME"] =  'httpService.getYNCHSYnhsdata';  //接口
    param["jybh"] = 'HISCCT001';
    Base.requestCct("queryData", param, function(data){
    	console.log("data: " + JSON.stringify(data));
    	if(data.flag == "1") {
			var chsParam = {};
			chsParam.request = data.result;
			chsParam.request.ynhsdata.infno = infno;
			chsParam.request.ynhsdata.dev_no = dev_no;
			chsParam.request.ynhsdata.input = verify;
			Base.console('身份核验入参', chsParam);
			ynhsa.chsApi(chsParam, function (ret_verify, err_verify) {
			    Base.console('身份核验-ret', ret_verify);
			    Base.console('身份核验-err', err_verify);
			    var success = ChsVerifySuccess(ret_verify, err_verify, infno);

			    if (success) {
			        Base.console('身份核验 成功 开始结算');
			        if (callback) {
			            Base.console('身份核验 成功 开始回调');
			            callback(ret_verify, err_verify, params);
			        } else {
			            Base.console('身份核验 成功 callback不存在');
			        }
			    } else {
			        if (failRepeat) {
			            Base.console('开始重新认证');
			                if (infno == '1192') {
			                    Base.console('开始重新认证-1191');
			                    readCardSuccessToAuth({dev_no: dev_no}, repeatParam.readCardResult, function (ret1, err1) {
			                        Base.console('重新认证回调 ret: ', ret1);
			                        Base.console('重新认证回调 err: ', err1);
			                        if (ChsReadCardAuthSuccess(ret1,  err1)) {
			                            verify.cardinfo.cardtoken = ret1.data.cardtoken;
			                            // 重新认证后需要更新 cardtoken
			                            params.cardtoken = ret1.data.cardtoken;
			                            ChsAuthenticateVerify(verify, infno, dev_no, callback, params, false);
			                        } else {
			                            Base.msgTip('身份认证失败，请将社保卡插入读卡器，重新点击结算' + ChsReadCardAuthFailMsg(ret1, err1), 5000);
			                        }
			                    }, null);
			                } else if (infno == '1194'){
			                    ChsIdNoAuth({dev_no: dev_no, aac002: repeatParam.aac002}, function (ret2, err2) {
			                        if (ChsIdNoAuthSuccess(ret2,  err2)) {
			                            verify.data.authtoken = ret2.data.authtoken;
			                            // 重新认证后需要更新 authtoken
			                            params.authtoken = ret2.data.authtoken;
			                            ChsAuthenticateVerify(verify, infno, dev_no, callback, params, false);
			                        } else {
			                            Base.msgTip('身份认证失败【1194】，请重新点击结算' + ChsReadCardAuthFailMsg(ret2, err2), 5000);
			                        }
			                    }, null);
			                }
			        } else {
			            Base.msgTip('身份核验失败【' + infno + '】，请重新点击结算: ' + ChsVerifyFailMsg(ret_verify, err_verify, infno), 5000);
			        }
			    }
			});
		}
    });
}


/**
 * chsApi 1192/1194 核验是否成功
 * @param ret chsApi身份核验返回ret
 * @param err chsApi身份核验返回err
 * @param infno 交易编号
 * @return {boolean}
 */
function ChsVerifySuccess(ret, err, infno) {
    var success = false;
    if (Base.isNotEmpty(ret)) {
        if (infno == '1192' && ret.pininfo.flag == '1') {
            success = true;
        } else if (infno == '1194' && ret.data.flag == '1') {
            success = true;
        }
    }
    return success;
}

/**
 * chsApi 1192/1194 错误信息获取
 * @param ret chsApi身份核验返回ret
 * @param err chsApi身份核验返回err
 * @param infno 交易编号
 * @return {string}
 */
function ChsVerifyFailMsg(ret, err, infno) {
    var errMsg = '未知错误';
    if (Base.isNotEmpty(err)) {
        errMsg = err.message
    }
    return errMsg;
}

/**
 * 打开蓝牙
 */
function openBluetooth() {
    if (!Base.isPos() && Base.server.USE_YNHSA) {
        // 打开蓝牙
        ynhsa.openBluetooth({type: "001"}, function (ret, err) {
            console.log('ynhsa.openBluetooth ret: ' + JSON.stringify(ret) + '\n err: ' + JSON.stringify(err));
        });
    }
}
