/**
 * 使用方法: 将此文件加载到页面中
 * 会获得 window.TSIGN_TOOLS 对象, 对象包含一个 md5 方法
 * const result = await window.TSIGN_TOOLS.md5(file)
 *
 * result 结构如下:
 * {
    fileSizeText: String, // 文件不足1kb, 值为xxxb; 超过1kb不足1mb, 值为xxxkb; 超过1mb, 值为xxxmb. 结果精确到小数点后两位
    fileSizeInByte: Number, // 文件实际大小, byge数, 即xxxb
    fileMD5: String,
    contentMD5: String,
 * }
 */

/* eslint-disable */
"use strict";

;(function () {

  function _slicedToArray(arr, i) { return _arrayWithHoles(arr) || _iterableToArrayLimit(arr, i) || _nonIterableRest(); }

  function _nonIterableRest() { throw new TypeError("Invalid attempt to destructure non-iterable instance"); }

  function _iterableToArrayLimit(arr, i) { if (!(Symbol.iterator in Object(arr) || Object.prototype.toString.call(arr) === "[object Arguments]")) { return; } var _arr = []; var _n = true; var _d = false; var _e = undefined; try { for (var _i = arr[Symbol.iterator](), _s; !(_n = (_s = _i.next()).done); _n = true) { _arr.push(_s.value); if (i && _arr.length === i) break; } } catch (err) { _d = true; _e = err; } finally { try { if (!_n && _i["return"] != null) _i["return"](); } finally { if (_d) throw _e; } } return _arr; }

  function _arrayWithHoles(arr) { if (Array.isArray(arr)) return arr; }

  //    md5_T[i] = parseInt(Math.abs(Math.sin(i)) * 4294967296.0);
  var MD5_T = [0x00000000, 0xd76aa478, 0xe8c7b756, 0x242070db, 0xc1bdceee, 0xf57c0faf, 0x4787c62a, 0xa8304613, 0xfd469501, 0x698098d8, 0x8b44f7af, 0xffff5bb1, 0x895cd7be, 0x6b901122, 0xfd987193, 0xa679438e, 0x49b40821, 0xf61e2562, 0xc040b340, 0x265e5a51, 0xe9b6c7aa, 0xd62f105d, 0x02441453, 0xd8a1e681, 0xe7d3fbc8, 0x21e1cde6, 0xc33707d6, 0xf4d50d87, 0x455a14ed, 0xa9e3e905, 0xfcefa3f8, 0x676f02d9, 0x8d2a4c8a, 0xfffa3942, 0x8771f681, 0x6d9d6122, 0xfde5380c, 0xa4beea44, 0x4bdecfa9, 0xf6bb4b60, 0xbebfbc70, 0x289b7ec6, 0xeaa127fa, 0xd4ef3085, 0x04881d05, 0xd9d4d039, 0xe6db99e5, 0x1fa27cf8, 0xc4ac5665, 0xf4292244, 0x432aff97, 0xab9423a7, 0xfc93a039, 0x655b59c3, 0x8f0ccc92, 0xffeff47d, 0x85845dd1, 0x6fa87e4f, 0xfe2ce6e0, 0xa3014314, 0x4e0811a1, 0xf7537e82, 0xbd3af235, 0x2ad7d2bb, 0xeb86d391];
  var MD5_round1 = [[0, 7, 1], [1, 12, 2], [2, 17, 3], [3, 22, 4], [4, 7, 5], [5, 12, 6], [6, 17, 7], [7, 22, 8], [8, 7, 9], [9, 12, 10], [10, 17, 11], [11, 22, 12], [12, 7, 13], [13, 12, 14], [14, 17, 15], [15, 22, 16]];
  var MD5_round2 = [[1, 5, 17], [6, 9, 18], [11, 14, 19], [0, 20, 20], [5, 5, 21], [10, 9, 22], [15, 14, 23], [4, 20, 24], [9, 5, 25], [14, 9, 26], [3, 14, 27], [8, 20, 28], [13, 5, 29], [2, 9, 30], [7, 14, 31], [12, 20, 32]];
  var MD5_round3 = [[5, 4, 33], [8, 11, 34], [11, 16, 35], [14, 23, 36], [1, 4, 37], [4, 11, 38], [7, 16, 39], [10, 23, 40], [13, 4, 41], [0, 11, 42], [3, 16, 43], [6, 23, 44], [9, 4, 45], [12, 11, 46], [15, 16, 47], [2, 23, 48]];
  var MD5_round4 = [[0, 6, 49], [7, 10, 50], [14, 15, 51], [5, 21, 52], [12, 6, 53], [3, 10, 54], [10, 15, 55], [1, 21, 56], [8, 6, 57], [15, 10, 58], [6, 15, 59], [13, 21, 60], [4, 6, 61], [11, 10, 62], [2, 15, 63], [9, 21, 64]];

  function MD5_F(x, y, z) {
    return x & y | ~x & z;
  }

  function MD5_G(x, y, z) {
    return x & z | y & ~z;
  }

  function MD5_H(x, y, z) {
    return x ^ y ^ z;
  }

  function MD5_I(x, y, z) {
    return y ^ (x | ~z);
  }

  var MD5_round = [[MD5_F, MD5_round1], [MD5_G, MD5_round2], [MD5_H, MD5_round3], [MD5_I, MD5_round4]];

  function MD5_pack(n32) {
    return String.fromCharCode(n32 & 0xff) + String.fromCharCode(n32 >>> 8 & 0xff) + String.fromCharCode(n32 >>> 16 & 0xff) + String.fromCharCode(n32 >>> 24 & 0xff);
  }

  function MD5_number(n) {
    while (n < 0) {
      n += 4294967296;
    }

    while (n > 4294967295) {
      n -= 4294967296;
    }

    return n;
  }

  function MD5_apply_round(x, s, f, abcd, r) {
    var _abcd = _slicedToArray(abcd, 4),
        a = _abcd[0],
        b = _abcd[1],
        c = _abcd[2],
        d = _abcd[3];

    var _r = _slicedToArray(r, 3),
        kk = _r[0],
        ss = _r[1],
        ii = _r[2];

    var u = f(s[b], s[c], s[d]);
    var t = s[a] + u + x[kk] + MD5_T[ii];
    t = MD5_number(t);
    t = t << ss | t >>> 32 - ss;
    t += s[b];
    s[a] = MD5_number(t);
  }

  function MD5_hash(data) {
    var i, j;
    var state = [0x67452301, 0xefcdab89, 0x98badcfe, 0x10325476];
    var len = data.length;
    var index = len & 0x3f;
    var padLen = index < 56 ? 56 - index : 120 - index;

    if (padLen > 0) {
      data += '\x80';

      for (i = 0; i < padLen - 1; i++) {
        data += '\x00';
      }
    }

    data += MD5_pack(len * 8);
    data += MD5_pack(0);
    len += padLen + 8;
    var abcd = [0, 1, 2, 3];
    var x = new Array(16);
    var s = new Array(4);

    for (var k = 0; k < len; k += 64) {
      for (i = 0, j = k; i < 16; i++, j += 4) {
        x[i] = data.charCodeAt(j) | data.charCodeAt(j + 1) << 8 | data.charCodeAt(j + 2) << 16 | data.charCodeAt(j + 3) << 24;
      }

      for (i = 0; i < 4; i++) {
        s[i] = state[i];
      }

      for (i = 0; i < 4; i++) {
        var _MD5_round$i = _slicedToArray(MD5_round[i], 2),
            f = _MD5_round$i[0],
            r = _MD5_round$i[1];

        for (j = 0; j < 16; j++) {
          MD5_apply_round(x, s, f, abcd, r[j]);
          var tmp = abcd[0];
          abcd[0] = abcd[3];
          abcd[3] = abcd[2];
          abcd[2] = abcd[1];
          abcd[1] = tmp;
        }
      }

      for (i = 0; i < 4; i++) {
        state[i] += s[i];
        state[i] = MD5_number(state[i]);
      }
    }

    return MD5_pack(state[0]) + MD5_pack(state[1]) + MD5_pack(state[2]) + MD5_pack(state[3]);
  }

  function MD5_hexhash(data) {
    var bit128 = MD5_hash(data);
    var out = '';

    for (var i = 0; i < 16; i++) {
      var c = bit128.charCodeAt(i);
      out += '0123456789abcdef'.charAt(c >> 4 & 0xf);
      out += '0123456789abcdef'.charAt(c & 0xf);
    }

    return out;
  }

  var K256 = [0x428a2f98, 0x71374491, 0xb5c0fbcf, 0xe9b5dba5,0x3956c25b, 0x59f111f1, 0x923f82a4, 0xab1c5ed5,0xd807aa98, 0x12835b01, 0x243185be, 0x550c7dc3,0x72be5d74, 0x80deb1fe, 0x9bdc06a7, 0xc19bf174,0xe49b69c1, 0xefbe4786, 0x0fc19dc6, 0x240ca1cc,0x2de92c6f, 0x4a7484aa, 0x5cb0a9dc, 0x76f988da,0x983e5152, 0xa831c66d, 0xb00327c8, 0xbf597fc7,0xc6e00bf3, 0xd5a79147, 0x06ca6351, 0x14292967,0x27b70a85, 0x2e1b2138, 0x4d2c6dfc, 0x53380d13,0x650a7354, 0x766a0abb, 0x81c2c92e, 0x92722c85,0xa2bfe8a1, 0xa81a664b, 0xc24b8b70, 0xc76c51a3,0xd192e819, 0xd6990624, 0xf40e3585, 0x106aa070,0x19a4c116, 0x1e376c08, 0x2748774c, 0x34b0bcb5,0x391c0cb3, 0x4ed8aa4a, 0x5b9cca4f, 0x682e6ff3,0x748f82ee, 0x78a5636f, 0x84c87814, 0x8cc70208,0x90befffa, 0xa4506ceb, 0xbef9a3f7, 0xc67178f2];
  
  function rotateRight(n, x) {
    return (x >>> n) | (x << (32 - n));
  }
  function choice(x, y, z) {
    return (x & y) ^ (~x & z);
  }
  function majority(x, y, z) {
    return (x & y) ^ (x & z) ^ (y & z);
  }
  function sha256_Sigma0(x) {
    return rotateRight(2, x) ^ rotateRight(13, x) ^ rotateRight(22, x);
  }
  function sha256_Sigma1(x) {
    return rotateRight(6, x) ^ rotateRight(11, x) ^ rotateRight(25, x);
  }
  function sha256_sigma0(x) {
    return rotateRight(7, x) ^ rotateRight(18, x) ^ (x >>> 3);
  }
  function sha256_sigma1(x) {
    return rotateRight(17, x) ^ rotateRight(19, x) ^ (x >>> 10);
  }
  function sha256_expand(W, j) {
    // eslint-disable-next-line no-return-assign
    return (W[j & 0x0f] +=
      sha256_sigma1(W[(j + 14) & 0x0f]) + W[(j + 9) & 0x0f] + sha256_sigma0(W[(j + 1) & 0x0f]));
  }
  var ihash, count, buffer;
  var sha256_hex_digits = '0123456789abcdef';
  function safe_add(x, y) {
    var lsw = (x & 0xffff) + (y & 0xffff);
    var msw = (x >> 16) + (y >> 16) + (lsw >> 16);
    return (msw << 16) | (lsw & 0xffff);
  }
  function sha256_init() {
    ihash = new Array(8);
    count = new Array(2);
    buffer = new Array(64);
    count[0] = count[1] = 0;
    ihash[0] = 0x6a09e667;
    ihash[1] = 0xbb67ae85;
    ihash[2] = 0x3c6ef372;
    ihash[3] = 0xa54ff53a;
    ihash[4] = 0x510e527f;
    ihash[5] = 0x9b05688c;
    ihash[6] = 0x1f83d9ab;
    ihash[7] = 0x5be0cd19;
  }
  
  function sha256_transform() {
    var a, b, c, d, e, f, g, h, T1, T2;
    var W = new Array(16);
  
    /* Initialize registers with the previous intermediate value */
    a = ihash[0];
    b = ihash[1];
    c = ihash[2];
    d = ihash[3];
    e = ihash[4];
    f = ihash[5];
    g = ihash[6];
    h = ihash[7];
  
    /* make 32-bit words */
    for (var i = 0; i < 16; i++) {
      W[i] =
        buffer[(i << 2) + 3] |
        (buffer[(i << 2) + 2] << 8) |
        (buffer[(i << 2) + 1] << 16) |
        (buffer[i << 2] << 24);
    }
  
    for (var j = 0; j < 64; j++) {
      T1 = h + sha256_Sigma1(e) + choice(e, f, g) + K256[j];
      if (j < 16) T1 += W[j];
      else T1 += sha256_expand(W, j);
      T2 = sha256_Sigma0(a) + majority(a, b, c);
      h = g;
      g = f;
      f = e;
      e = safe_add(d, T1);
      d = c;
      c = b;
      b = a;
      a = safe_add(T1, T2);
    }
  
    /* Compute the current intermediate hash value */
    ihash[0] += a;
    ihash[1] += b;
    ihash[2] += c;
    ihash[3] += d;
    ihash[4] += e;
    ihash[5] += f;
    ihash[6] += g;
    ihash[7] += h;
  }
  
  function sha256_update(data, inputLen) {
    var i,index,curpos = 0;
  
    /* Compute number of bytes mod 64 */
    index = (count[0] >> 3) & 0x3f;
    var remainder = inputLen & 0x3f;
  
    /* Update number of bits */
    if ((count[0] += inputLen << 3) < inputLen << 3) count[1]++;
    count[1] += inputLen >> 29;
  
    /* Transform as many times as possible */
    for (i = 0; i + 63 < inputLen; i += 64) {
      for (var j = index; j < 64; j++) {
        buffer[j] = data.charCodeAt(curpos++);
      }
      sha256_transform();
      index = 0;
    }
  
    /* Buffer remaining input */
    for (var j = 0; j < remainder; j++) buffer[j] = data.charCodeAt(curpos++);
  }
  
  function sha256_final() {
    var index = (count[0] >> 3) & 0x3f;
    buffer[index++] = 0x80;
    if (index <= 56) {
      for (var i = index; i < 56; i++) buffer[i] = 0;
    } else {
      for (var i = index; i < 64; i++) buffer[i] = 0;
      sha256_transform();
      for (var i = 0; i < 56; i++) buffer[i] = 0;
    }
    buffer[56] = (count[1] >>> 24) & 0xff;
    buffer[57] = (count[1] >>> 16) & 0xff;
    buffer[58] = (count[1] >>> 8) & 0xff;
    buffer[59] = count[1] & 0xff;
    buffer[60] = (count[0] >>> 24) & 0xff;
    buffer[61] = (count[0] >>> 16) & 0xff;
    buffer[62] = (count[0] >>> 8) & 0xff;
    buffer[63] = count[0] & 0xff;
    sha256_transform();
  }
  
  function sha256_encode_hex() {
    var output = '';
    for (var i = 0; i < 8; i++) {
      for (var j = 28; j >= 0; j -= 4) output += sha256_hex_digits.charAt((ihash[i] >>> j) & 0x0f);
    }
    return output;
  }
  
  function sha256_digest(data) {
    sha256_init();
    sha256_update(data, data.length);
    sha256_final();
    return sha256_encode_hex();
  }

  var BIT_BER_KB = 1024;
  var BIT_PER_MB = 1024 * 1024;

  function getFileSize(file) {
    var size = file.size;
    if (size < BIT_BER_KB) return size.toFixed(2) + 'b';
    if (size < BIT_PER_MB) return (size / BIT_BER_KB).toFixed(2) + 'kb';
    return (size / BIT_PER_MB).toFixed(2) + 'mb';
  }

  function md5Fun(file) {
    return new Promise(function (resolve, reject) {
      try {
        var reader = new FileReader(); // 将文件以二进制形式读入

        reader.readAsBinaryString(file);

        reader.onload = function (f) {
          try {
            var fileblob = f.target.result;
            var fileSizeText = getFileSize(file);
            var md5Bit128 = MD5_hash(fileblob);
            var fileMD5 = MD5_hexhash(fileblob);
            var contentMD5 = btoa(md5Bit128);
            var fileSizeInByte = file.size;
            var fileSHA256 = sha256_digest(fileblob);
            resolve({
              fileSizeText: fileSizeText,
              fileSizeInByte: fileSizeInByte,
              fileMD5: fileMD5,
              fileSHA256: fileSHA256,
              contentMD5: contentMD5
            });
          } catch (e) {
            reject(e);
          }
        };
      } catch (e) {
        reject(e);
      }
    });
  }

  window.TSIGN_TOOLS = window.TSIGN_TOOLS || {};
  window.TSIGN_TOOLS.md5 = md5Fun;
})();
